@@ -1,4 +1,4 @@
-/*	$OpenBSD: extern.h,v 1.192 2023/09/25 14:56:20 tb Exp $ */
+/*	$OpenBSD: extern.h,v 1.196 2023/12/11 19:05:20 job Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -210,6 +210,7 @@ struct mft {
 	char		*sia; /* SIA signedObject */
 	char		*ski; /* SKI */
 	char		*crl; /* CRL file name */
+	unsigned char	 mfthash[SHA256_DIGEST_LENGTH];
 	unsigned char	 crlhash[SHA256_DIGEST_LENGTH];
 	time_t		 signtime; /* CMS signing-time attribute */
 	time_t		 thisupdate; /* from the eContent */
@@ -432,6 +433,7 @@ RB_PROTOTYPE(brk_tree, brk, entry, brkcmp);
 struct crl {
 	RB_ENTRY(crl)	 entry;
 	char		*aki;
+	char		*number;
 	X509_CRL	*x509_crl;
 	time_t		 lastupdate;	/* do not use before */
 	time_t		 nextupdate;	/* do not use after */
@@ -613,7 +615,7 @@ struct tal	*tal_read(struct ibuf *);
 void		 cert_buffer(struct ibuf *, const struct cert *);
 void		 cert_free(struct cert *);
 void		 auth_tree_free(struct auth_tree *);
-struct cert	*cert_parse_ee_cert(const char *, X509 *);
+struct cert	*cert_parse_ee_cert(const char *, int, X509 *);
 struct cert	*cert_parse_pre(const char *, const unsigned char *, size_t);
 struct cert	*cert_parse(const char *, struct cert *);
 struct cert	*ta_parse(const char *, struct cert *, const unsigned char *,
@@ -712,11 +714,12 @@ void		 ip_addr_range_print(const struct ip_addr_range *, enum afi,
 			char *, size_t);
 int		 ip_addr_cmp(const struct ip_addr *, const struct ip_addr *);
 int		 ip_addr_check_overlap(const struct cert_ip *,
-			const char *, const struct cert_ip *, size_t);
+			const char *, const struct cert_ip *, size_t, int);
 int		 ip_addr_check_covered(enum afi, const unsigned char *,
 			const unsigned char *, const struct cert_ip *, size_t);
 int		 ip_cert_compose_ranges(struct cert_ip *);
 void		 ip_roa_compose_ranges(struct roa_ip *);
+void		 ip_warn(const char *, const struct cert_ip *, const char *);
 
 int		 sbgp_addr(const char *, struct cert_ip *, size_t *,
 		    enum afi, const ASN1_BIT_STRING *);
@@ -730,9 +733,10 @@ int		 sbgp_parse_ipaddrblk(const char *, const IPAddrBlocks *,
 
 int		 as_id_parse(const ASN1_INTEGER *, uint32_t *);
 int		 as_check_overlap(const struct cert_as *, const char *,
-			const struct cert_as *, size_t);
+			const struct cert_as *, size_t, int);
 int		 as_check_covered(uint32_t, uint32_t,
 			const struct cert_as *, size_t);
+void		 as_warn(const char *, const struct cert_as *, const char *);
 
 int		 sbgp_as_id(const char *, struct cert_as *, size_t *,
 		    const ASN1_INTEGER *);
@@ -742,6 +746,12 @@ int		 sbgp_as_range(const char *, struct cert_as *, size_t *,
 int		 sbgp_parse_assysnum(const char *, const ASIdentifiers *,
 		    struct cert_as **, size_t *);
 
+/* Constraints-specific */
+void		 constraints_load(void);
+void		 constraints_unload(void);
+void		 constraints_parse(void);
+int		 constraints_validate(const char *, const struct cert *);
+
 /* Parser-specific */
 void		 entity_free(struct entity *);
 void		 entity_read_req(struct ibuf *, struct entity *);
@@ -839,6 +849,7 @@ int		 x509_get_notbefore(X509 *, const char *, time_t *);
 int		 x509_get_notafter(X509 *, const char *, time_t *);
 int		 x509_get_crl(X509 *, const char *, char **);
 char		*x509_crl_get_aki(X509_CRL *, const char *);
+char		*x509_crl_get_number(X509_CRL *, const char *);
 char		*x509_get_pubkey(X509 *, const char *);
 enum cert_purpose	 x509_get_purpose(X509 *, const char *);
 int		 x509_get_time(const ASN1_TIME *, time_t *);
@@ -864,6 +875,10 @@ void		 aspa_print(const X509 *, const struct aspa *);
 void		 tak_print(const X509 *, const struct tak *);
 void		 geofeed_print(const X509 *, const struct geofeed *);
 
+/* Missing RFC 3779 API */
+IPAddrBlocks *IPAddrBlocks_new(void);
+void IPAddrBlocks_free(IPAddrBlocks *);
+
 /* Output! */
 
 extern int	 outformats;
@@ -933,7 +948,8 @@ int	mkpathat(int, const char *);
 /* Maximum acceptable URI length */
 #define MAX_URI_LENGTH		2048
 
-/* Maximum acceptable file size */
+/* Min/Max acceptable file size */
+#define MIN_FILE_SIZE		100
 #define MAX_FILE_SIZE		4000000
 
 /* Maximum number of FileNameAndHash entries per RSC checklist. */
