@@ -1,4 +1,4 @@
-/*	$OpenBSD: parser.c,v 1.99 2023/09/25 11:08:45 tb Exp $ */
+/*	$OpenBSD: parser.c,v 1.104 2023/12/18 23:42:20 job Exp $ */
 /*
  * Copyright (c) 2019 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -280,6 +280,10 @@ proc_parser_mft_pre(struct entity *entp, enum location loc, char **file,
 		free(der);
 		return NULL;
 	}
+
+	if (!EVP_Digest(der, len, mft->mfthash, NULL, EVP_sha256(), NULL))
+		errx(1, "EVP_Digest failed");
+
 	free(der);
 
 	*crl = parse_load_crl_from_mft(entp, mft, DIR_TEMP, crlfile);
@@ -310,7 +314,7 @@ proc_parser_mft_pre(struct entity *entp, enum location loc, char **file,
  */
 static struct mft *
 proc_parser_mft_post(char *file, struct mft *mft, const char *path,
-    const char *errstr)
+    const char *errstr, int *warned)
 {
 	/* check that now is not before from */
 	time_t now = get_current_time();
@@ -318,6 +322,8 @@ proc_parser_mft_post(char *file, struct mft *mft, const char *path,
 	if (mft == NULL) {
 		if (errstr == NULL)
 			errstr = "no valid mft available";
+		if ((*warned)++ > 0)
+			return NULL;
 		warnx("%s: %s", file, errstr);
 		return NULL;
 	}
@@ -359,13 +365,14 @@ proc_parser_mft(struct entity *entp, struct mft **mp, char **crlfile,
 	struct crl	*crl, *crl1, *crl2;
 	char		*file, *file1, *file2, *crl1file, *crl2file;
 	const char	*err1, *err2;
+	int		 r, warned = 0;
 
 	*mp = NULL;
 	*crlmtime = 0;
 
-	mft1 = proc_parser_mft_pre(entp, DIR_VALID, &file1, &crl1, &crl1file,
+	mft1 = proc_parser_mft_pre(entp, DIR_TEMP, &file1, &crl1, &crl1file,
 	    &err1);
-	mft2 = proc_parser_mft_pre(entp, DIR_TEMP, &file2, &crl2, &crl2file,
+	mft2 = proc_parser_mft_pre(entp, DIR_VALID, &file2, &crl2, &crl2file,
 	    &err2);
 
 	/* overload error from temp file if it is set */
@@ -373,21 +380,48 @@ proc_parser_mft(struct entity *entp, struct mft **mp, char **crlfile,
 		if (err2 != NULL)
 			err1 = err2;
 
-	if (mft_compare(mft1, mft2) == 1) {
+	r = mft_compare(mft1, mft2);
+	if (r == -1 && mft1 != NULL && mft2 != NULL)
+		warnx("%s: unexpected manifest number (want >= #%s, got #%s)",
+		    file1, mft2->seqnum, mft1->seqnum);
+
+	if (r == 0 && memcmp(mft1->mfthash, mft2->mfthash,
+	    SHA256_DIGEST_LENGTH) != 0)
+		warnx("%s: manifest misissuance, #%s was recycled",
+		    file1, mft1->seqnum);
+
+	if (r == 1) {
+		*mp = proc_parser_mft_post(file1, mft1, entp->path, err1,
+		    &warned);
+		if (*mp == NULL) {
+			if (mft2 != NULL)
+				warnx("%s#%s: failed fetch, continuing with #%s"
+				    " from cache", file1, mft1->seqnum,
+				    mft2->seqnum);
+			mft1 = NULL;
+		}
+	}
+
+	if (*mp != NULL) {
 		mft_free(mft2);
 		crl_free(crl2);
 		free(crl2file);
 		free(file2);
-		*mp = proc_parser_mft_post(file1, mft1, entp->path, err1);
+
 		crl = crl1;
 		file = file1;
 		*crlfile = crl1file;
 	} else {
+		if (err2 == NULL)
+			err2 = err1;
+		*mp = proc_parser_mft_post(file2, mft2, entp->path, err2,
+		    &warned);
+
 		mft_free(mft1);
 		crl_free(crl1);
 		free(crl1file);
 		free(file1);
-		*mp = proc_parser_mft_post(file2, mft2, entp->path, err2);
+
 		crl = crl2;
 		file = file2;
 		*crlfile = crl2file;
@@ -441,6 +475,13 @@ proc_parser_cert(char *file, const unsigned char *der, size_t len,
 
 	cert->talid = a->cert->talid;
 
+	if (cert->purpose == CERT_PURPOSE_BGPSEC_ROUTER) {
+		if (!constraints_validate(file, cert)) {
+			cert_free(cert);
+			return NULL;
+		}
+	}
+
 	/*
 	 * Add validated CA certs to the RPKI auth tree.
 	 */
@@ -813,6 +854,7 @@ proc_parser(int fd)
 	OpenSSL_add_all_ciphers();
 	OpenSSL_add_all_digests();
 	x509_init_oid();
+	constraints_parse();
 
 	if ((ctx = X509_STORE_CTX_new()) == NULL)
 		err(1, "X509_STORE_CTX_new");
