@@ -1,4 +1,4 @@
-/*	$OpenBSD: repo.c,v 1.44 2023/04/26 16:32:41 claudio Exp $ */
+/*	$OpenBSD: repo.c,v 1.51 2023/07/20 05:18:31 claudio Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -119,6 +119,7 @@ static void		 remove_contents(char *);
 struct filepath {
 	RB_ENTRY(filepath)	entry;
 	char			*file;
+	time_t			 mtime;
 };
 
 static inline int
@@ -133,12 +134,13 @@ RB_PROTOTYPE(filepath_tree, filepath, entry, filepathcmp);
  * Functions to lookup which files have been accessed during computation.
  */
 int
-filepath_add(struct filepath_tree *tree, char *file)
+filepath_add(struct filepath_tree *tree, char *file, time_t mtime)
 {
 	struct filepath *fp;
 
 	if ((fp = malloc(sizeof(*fp))) == NULL)
 		err(1, NULL);
+	fp->mtime = mtime;
 	if ((fp->file = strdup(file)) == NULL)
 		err(1, NULL);
 
@@ -617,22 +619,26 @@ repo_alloc(int talid)
  * Parse the RRDP state file if it exists and set the session struct
  * based on that information.
  */
-static void
-rrdp_parse_state(const struct rrdprepo *rr, struct rrdp_session *state)
+static struct rrdp_session *
+rrdp_session_parse(const struct rrdprepo *rr)
 {
 	FILE *f;
-	int fd, ln = 0;
+	struct rrdp_session *state;
+	int fd, ln = 0, deltacnt = 0;
 	const char *errstr;
 	char *line = NULL, *file;
 	size_t len = 0;
 	ssize_t n;
 
+	if ((state = calloc(1, sizeof(*state))) == NULL)
+		err(1, NULL);
+
 	file = rrdp_state_filename(rr, 0);
 	if ((fd = open(file, O_RDONLY)) == -1) {
 		if (errno != ENOENT)
 			warn("%s: open state file", rr->basedir);
 		free(file);
-		return;
+		return state;
 	}
 	free(file);
 	f = fdopen(fd, "r");
@@ -653,39 +659,47 @@ rrdp_parse_state(const struct rrdprepo *rr, struct rrdp_session *state)
 				goto fail;
 			break;
 		case 2:
+			if (strcmp(line, "-") == 0)
+				break;
 			if ((state->last_mod = strdup(line)) == NULL)
 				err(1, NULL);
 			break;
 		default:
-			goto fail;
+			if (deltacnt >= MAX_RRDP_DELTAS)
+				goto fail;
+			if ((state->deltas[deltacnt++] = strdup(line)) == NULL)
+				err(1, NULL);
+			break;
 		}
 		ln++;
 	}
 
-	free(line);
 	if (ferror(f))
 		goto fail;
 	fclose(f);
-	return;
+	free(line);
+	return state;
 
-fail:
+ fail:
 	warnx("%s: troubles reading state file", rr->basedir);
 	fclose(f);
+	free(line);
 	free(state->session_id);
 	free(state->last_mod);
 	memset(state, 0, sizeof(*state));
+	return state;
 }
 
 /*
  * Carefully write the RRDP session state file back.
  */
 void
-rrdp_save_state(unsigned int id, struct rrdp_session *state)
+rrdp_session_save(unsigned int id, struct rrdp_session *state)
 {
 	struct rrdprepo *rr;
 	char *temp, *file;
-	FILE *f;
-	int fd;
+	FILE *f = NULL;
+	int fd, i;
 
 	rr = rrdp_find(id);
 	if (rr == NULL)
@@ -694,10 +708,8 @@ rrdp_save_state(unsigned int id, struct rrdp_session *state)
 	file = rrdp_state_filename(rr, 0);
 	temp = rrdp_state_filename(rr, 1);
 
-	if ((fd = mkostemp(temp, O_CLOEXEC)) == -1) {
-		warn("mkostemp %s", temp);
+	if ((fd = mkostemp(temp, O_CLOEXEC)) == -1)
 		goto fail;
-	}
 	(void)fchmod(fd, 0644);
 	f = fdopen(fd, "w");
 	if (f == NULL)
@@ -705,37 +717,94 @@ rrdp_save_state(unsigned int id, struct rrdp_session *state)
 
 	/* write session state file out */
 	if (fprintf(f, "%s\n%lld\n", state->session_id,
-	    state->serial) < 0) {
-		fclose(f);
-		goto fail;
-	}
-	if (state->last_mod != NULL) {
-		if (fprintf(f, "%s\n", state->last_mod) < 0) {
-			fclose(f);
-			goto fail;
-		}
-	}
-	if (fclose(f) != 0)
+	    state->serial) < 0)
 		goto fail;
 
-	if (rename(temp, file) == -1)
-		warn("%s: rename state file", rr->basedir);
+	if (state->last_mod != NULL) {
+		if (fprintf(f, "%s\n", state->last_mod) < 0)
+			goto fail;
+	} else {
+		if (fprintf(f, "-\n") < 0)
+			goto fail;
+	}
+	for (i = 0; i < MAX_RRDP_DELTAS && state->deltas[i] != NULL; i++) {
+		if (fprintf(f, "%s\n", state->deltas[i]) < 0)
+			goto fail;
+	}
+	if (fclose(f) != 0) {
+		f = NULL;
+		goto fail;
+	}
+
+	if (rename(temp, file) == -1) {
+		warn("%s: rename %s to %s", rr->basedir, temp, file);
+		unlink(temp);
+	}
 
 	free(temp);
 	free(file);
 	return;
 
-fail:
-	warnx("%s: failed to save state", rr->basedir);
+ fail:
+	warn("%s: save state to %s", rr->basedir, temp);
+	if (f != NULL)
+		fclose(f);
 	unlink(temp);
 	free(temp);
 	free(file);
 }
 
+/*
+ * Free an rrdp_session pointer. Safe to call with NULL.
+ */
+void
+rrdp_session_free(struct rrdp_session *s)
+{
+	size_t i;
+
+	if (s == NULL)
+		return;
+	free(s->session_id);
+	free(s->last_mod);
+	for (i = 0; i < sizeof(s->deltas) / sizeof(s->deltas[0]); i++)
+		free(s->deltas[i]);
+	free(s);
+}
+
+void
+rrdp_session_buffer(struct ibuf *b, const struct rrdp_session *s)
+{
+	size_t i;
+
+	io_str_buffer(b, s->session_id);
+	io_simple_buffer(b, &s->serial, sizeof(s->serial));
+	io_str_buffer(b, s->last_mod);
+	for (i = 0; i < sizeof(s->deltas) / sizeof(s->deltas[0]); i++)
+		io_str_buffer(b, s->deltas[i]);
+}
+
+struct rrdp_session *
+rrdp_session_read(struct ibuf *b)
+{
+	struct rrdp_session *s;
+	size_t i;
+
+	if ((s = calloc(1, sizeof(*s))) == NULL)
+		err(1, NULL);
+
+	io_read_str(b, &s->session_id);
+	io_read_buf(b, &s->serial, sizeof(s->serial));
+	io_read_str(b, &s->last_mod);
+	for (i = 0; i < sizeof(s->deltas) / sizeof(s->deltas[0]); i++)
+		io_read_str(b, &s->deltas[i]);
+
+	return s;
+}
+
 static struct rrdprepo *
 rrdp_get(const char *uri)
 {
-	struct rrdp_session state = { 0 };
+	struct rrdp_session *state;
 	struct rrdprepo *rr;
 
 	SLIST_FOREACH(rr, &rrdprepos, entry)
@@ -765,10 +834,9 @@ rrdp_get(const char *uri)
 	}
 
 	/* parse state and start the sync */
-	rrdp_parse_state(rr, &state);
-	rrdp_fetch(rr->id, rr->notifyuri, rr->notifyuri, &state);
-	free(state.session_id);
-	free(state.last_mod);
+	state = rrdp_session_parse(rr);
+	rrdp_fetch(rr->id, rr->notifyuri, rr->notifyuri, state);
+	rrdp_session_free(state);
 
 	logx("%s: pulling from %s", rr->notifyuri, "network");
 
@@ -805,7 +873,8 @@ rrdp_handle_file(unsigned int id, enum publish_type pt, char *uri,
 	struct filepath *fp;
 	ssize_t s;
 	char *fn = NULL;
-	int fd = -1, try = 0;
+	int fd = -1, try = 0, deleted = 0;
+	int flags;
 
 	rr = rrdp_find(id);
 	if (rr == NULL)
@@ -837,11 +906,13 @@ rrdp_handle_file(unsigned int id, enum publish_type pt, char *uri,
 
 	/* write new content or mark uri as deleted. */
 	if (pt == PUB_DEL) {
-		filepath_add(&rr->deleted, uri);
+		filepath_add(&rr->deleted, uri, 0);
 	} else {
 		fp = filepath_find(&rr->deleted, uri);
-		if (fp != NULL)
+		if (fp != NULL) {
 			filepath_put(&rr->deleted, fp);
+			deleted = 1;
+		}
 
 		/* add new file to rrdp dir */
 		if ((fn = rrdp_filename(rr, uri, 0)) == NULL)
@@ -850,8 +921,17 @@ rrdp_handle_file(unsigned int id, enum publish_type pt, char *uri,
 		if (repo_mkpath(AT_FDCWD, fn) == -1)
 			goto fail;
 
-		fd = open(fn, O_WRONLY|O_CREAT|O_TRUNC, 0644);
+		flags = O_WRONLY|O_CREAT|O_TRUNC;
+		if (pt == PUB_ADD && !deleted)
+			flags |= O_EXCL;
+		fd = open(fn, flags, 0644);
 		if (fd == -1) {
+			if (errno == EEXIST) {
+				warnx("%s: duplicate publish element for %s",
+				    rr->notifyuri, fn);
+				free(fn);
+				return 0;
+			}
 			warn("open %s", fn);
 			goto fail;
 		}
@@ -1220,7 +1300,7 @@ repo_proto(const struct repo *rp)
 
 	if (rp->ta != NULL) {
 		const struct tarepo *tr = rp->ta;
-		if (tr->uriidx < tr->urisz && 
+		if (tr->uriidx < tr->urisz &&
 		    strncasecmp(tr->uri[tr->uriidx], "rsync://", 8) == 0)
 			return "rsync";
 		else
@@ -1395,15 +1475,9 @@ repo_stat_inc(struct repo *rp, int talid, enum rtype type, enum stype subtype)
 		case STYPE_DEC_UNIQUE:
 			rp->stats[talid].vaps_uniqs--;
 			break;
-		case STYPE_BOTH:
+		case STYPE_PROVIDERS:
 			rp->stats[talid].vaps_pas++;
 			break;
-		case STYPE_ONLY_IPV4:
-			rp->stats[talid].vaps_pas4++;
-			break;
-		case STYPE_ONLY_IPV6:
-			rp->stats[talid].vaps_pas6++;
-			break;
 		default:
 			break;
 		}
@@ -1526,6 +1600,28 @@ repo_move_valid(struct filepath_tree *tree)
 			base = strchr(fp->file + rrdpsz, '/');
 			assert(base != NULL);
 			fn = base + 1;
+
+			/*
+			 * Adjust file last modification time in order to
+			 * minimize RSYNC synchronization load after transport
+			 * failover.
+			 * While serializing RRDP datastructures to disk, set
+			 * the last modified timestamp to the CMS signing-time,
+			 * the X.509 notBefore, or CRL lastUpdate timestamp.
+			 */
+			if (fp->mtime != 0) {
+				int ret;
+				struct timespec ts[2];
+
+				ts[0].tv_nsec = UTIME_OMIT;
+				ts[1].tv_sec = fp->mtime;
+				ts[1].tv_nsec = 0;
+				ret = utimensat(AT_FDCWD, fp->file, ts, 0);
+				if (ret == -1) {
+					warn("utimensat %s", fp->file);
+					continue;
+				}
+			}
 		}
 
 		if (repo_mkpath(AT_FDCWD, fn) == -1)
@@ -1630,17 +1726,17 @@ repo_cleanup_entry(FTSENT *e, struct filepath_tree *tree, int cachefd)
 			} else if (fts_state.type == RSYNC_DIR) {
 				/* no need to keep rsync files */
 				if (verbose > 1)
-					logx("superfluous %s", path);
+					logx("deleted superfluous %s", path);
 				if (fts_state.rp != NULL)
 					fts_state.rp->repostats.del_extra_files++;
-				else 
+				else
 					stats.repo_stats.del_extra_files++;
 			} else {
 				if (verbose > 1)
 					logx("deleted %s", path);
 				if (fts_state.rp != NULL)
 					fts_state.rp->repostats.del_files++;
-				else 
+				else
 					stats.repo_stats.del_files++;
 			}
 		}
@@ -1649,28 +1745,32 @@ repo_cleanup_entry(FTSENT *e, struct filepath_tree *tree, int cachefd)
 		if (e->fts_level == FTS_ROOTLEVEL)
 			fts_state.type = BASE_DIR;
 		if (e->fts_level == 1) {
+			/* rpki.example.org or .rrdp / .rsync */
 			if (strcmp(".rsync", e->fts_name) == 0) {
 				fts_state.type = RSYNC_DIR;
 				fts_state.rp = NULL;
 			} else if (strcmp(".rrdp", e->fts_name) == 0) {
 				fts_state.type = RRDP_DIR;
 				fts_state.rp = NULL;
-			} else {
-				fts_state.type = BASE_DIR;
-				fts_state.rp = repo_bypath(path);
 			}
 		}
 		if (e->fts_level == 2) {
-			if (fts_state.type == RSYNC_DIR)
+			/* rpki.example.org/repository or .rrdp/hashdir */
+			if (fts_state.type == BASE_DIR)
 				fts_state.rp = repo_bypath(path);
 			/*
 			 * special handling for rrdp directories,
 			 * clear them if they are not used anymore but
 			 * only if rrdp is active.
+			 * Look them up just using the hash.
 			 */
 			if (fts_state.type == RRDP_DIR)
 				fts_state.rp = repo_rrdp_bypath(path);
 		}
+		if (e->fts_level == 3 && fts_state.type == RSYNC_DIR) {
+			/* .rsync/rpki.example.org/repository */
+			fts_state.rp = repo_bypath(path + strlen(".rsync/"));
+		}
 		break;
 	case FTS_DP:
 		if (e->fts_level == FTS_ROOTLEVEL)
@@ -1690,7 +1790,7 @@ repo_cleanup_entry(FTSENT *e, struct filepath_tree *tree, int cachefd)
 				warn("rmdir %s", path);
 			if (fts_state.rp != NULL)
 				fts_state.rp->repostats.del_dirs++;
-			else 
+			else
 				stats.repo_stats.del_dirs++;
 		}
 		break;
@@ -1757,7 +1857,8 @@ repo_free(void)
 }
 
 /*
- * Remove all files and directories under base but do not remove base itself.
+ * Remove all files and directories under base.
+ * Do not remove base directory itself and the .state file.
  */
 static void
 remove_contents(char *base)
@@ -1774,6 +1875,9 @@ remove_contents(char *base)
 		case FTS_NSOK:
 		case FTS_SL:
 		case FTS_SLNONE:
+			if (e->fts_level == 1 &&
+			    strcmp(e->fts_name, ".state") == 0)
+				break;
 			if (unlink(e->fts_accpath) == -1)
 				warn("unlink %s", e->fts_path);
 			break;
