@@ -1,4 +1,4 @@
-/*	$OpenBSD: roa.c,v 1.70 2023/09/25 11:08:45 tb Exp $ */
+/*	$OpenBSD: roa.c,v 1.72 2023/12/14 07:52:53 tb Exp $ */
 /*
  * Copyright (c) 2022 Theo Buehler <tb@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -104,7 +104,7 @@ roa_parse_econtent(const unsigned char *d, size_t dsz, struct parse *p)
 	RouteOriginAttestation		*roa;
 	const ROAIPAddressFamily	*addrfam;
 	const STACK_OF(ROAIPAddress)	*addrs;
-	int				 addrsz;
+	int				 addrsz, ipv4_seen = 0, ipv6_seen = 0;
 	enum afi			 afi;
 	const ROAIPAddress		*addr;
 	uint64_t			 maxlen;
@@ -129,8 +129,8 @@ roa_parse_econtent(const unsigned char *d, size_t dsz, struct parse *p)
 	}
 
 	ipaddrblocksz = sk_ROAIPAddressFamily_num(roa->ipAddrBlocks);
-	if (ipaddrblocksz > 2) {
-		warnx("%s: draft-rfc6482bis: too many ipAddrBlocks "
+	if (ipaddrblocksz != 1 && ipaddrblocksz != 2) {
+		warnx("%s: draft-rfc6482bis: unexpected number of ipAddrBlocks "
 		    "(got %d, expected 1 or 2)", p->fn, ipaddrblocksz);
 		goto out;
 	}
@@ -146,6 +146,29 @@ roa_parse_econtent(const unsigned char *d, size_t dsz, struct parse *p)
 			goto out;
 		}
 
+		switch(afi) {
+		case AFI_IPV4:
+			if (ipv4_seen++ > 0) {
+				warnx("%s: RFC 6482bis section 4.3.2: "
+				    "IPv4 appears twice", p->fn);
+				goto out;
+			}
+			break;
+		case AFI_IPV6:
+			if (ipv6_seen++ > 0) {
+				warnx("%s: RFC 6482bis section 4.3.2: "
+				    "IPv6 appears twice", p->fn);
+				goto out;
+			}
+			break;
+		}
+
+		if (addrsz == 0) {
+			warnx("%s: RFC 6482bis, section 4.3.2: "
+			    "empty ROAIPAddressFamily", p->fn);
+			goto out;
+		}
+
 		if (p->res->ipsz + addrsz >= MAX_IP_SIZE) {
 			warnx("%s: too many ROAIPAddress entries: limit %d",
 			    p->fn, MAX_IP_SIZE);
@@ -257,7 +280,7 @@ roa_parse(X509 **x509, const char *fn, int talid, const unsigned char *der,
 		goto out;
 	}
 
-	if ((cert = cert_parse_ee_cert(fn, *x509)) == NULL)
+	if ((cert = cert_parse_ee_cert(fn, talid, *x509)) == NULL)
 		goto out;
 
 	if (cert->asz > 0) {
